PYTHONPATH_ENV_VAR: str = "PYTHONPATH"
AGNO_RUNTIME_ENV_VAR: str = "AGNO_RUNTIME"

AGNO_INFRA_NAME: str = "AGNO_INFRA_NAME"
AGNO_INFRA_ROOT: str = "AGNO_INFRA_ROOT"
AGNO_INFRA_DIR: str = "AGNO_INFRA_DIR"
REQUIREMENTS_FILE_PATH_ENV_VAR: str = "REQUIREMENTS_FILE_PATH"

AWS_REGION_ENV_VAR: str = "AWS_REGION"
AWS_DEFAULT_REGION_ENV_VAR: str = "AWS_DEFAULT_REGION"
AWS_PROFILE_ENV_VAR: str = "AWS_PROFILE"
